#########################################
# Author: Serge Kozlov skozlov@mysql.com
# Date: 07/10/2006
# Purpose: Delete all rows from work tables (t1, t2) on master
# Requirements: 
#########################################

DELETE FROM t1;
DELETE FROM t2;
